@@ -442,6 +442,16 @@ def pull_objects(hrefs_missing, hrefs_modified, conn, syncdb, etagdict):
             die_atnode('Missing href.', node)
         href = hrefnode.text
 
+        statusnode = node.find("./D:status", namespaces=nsmap)
+        if statusnode is not None:
+            status = re.match(r'HTTP.*(\d\d\d)', statusnode.text)
+            if status is None:
+                die_atnode('Could not parse status.', node)
+            statuscode = status.group(1)
+            if statuscode == '404':
+                print('Skipping missing item: {}'.format(href))
+                continue
+
         etagnode = node.find("./D:propstat/D:prop/D:getetag", namespaces=nsmap)
         if etagnode is None:
             die_atnode('Missing ETag.', node)
@@ -509,11 +519,21 @@ def run_hook(name):
 nsmap = {"D": "DAV:", "C": "urn:ietf:params:xml:ns:caldav"}
 
 # Initialize default values.
-configfn = os.path.expanduser("~/.calcurse/caldav/config")
-lockfn = os.path.expanduser("~/.calcurse/caldav/lock")
-syncdbfn = os.path.expanduser("~/.calcurse/caldav/sync.db")
-hookdir = os.path.expanduser("~/.calcurse/caldav/hooks/")
-oauth_file = os.path.expanduser("~/.calcurse/caldav/oauth2_cred")
+if os.path.isdir(os.path.expanduser("~/.calcurse")):
+    configfn = os.path.expanduser("~/.calcurse/caldav/config")
+    lockfn = os.path.expanduser("~/.calcurse/caldav/lock")
+    syncdbfn = os.path.expanduser("~/.calcurse/caldav/sync.db")
+    hookdir = os.path.expanduser("~/.calcurse/caldav/hooks/")
+    oauth_file = os.path.expanduser("~/.calcurse/caldav/oauth2_cred")
+else:
+    calcurse_data = os.getenv("XDG_DATA_HOME", os.path.expanduser("~/.local/share")) + "/calcurse"
+    calcurse_config = os.getenv("XDG_CONFIG_HOME", os.path.expanduser("~/.config")) + "/calcurse"
+
+    configfn = os.path.expanduser(calcurse_config + "/caldav/config")
+    lockfn = os.path.expanduser(calcurse_data + "/caldav/lock")
+    syncdbfn = os.path.expanduser(calcurse_data + "/caldav/sync.db")
+    hookdir = os.path.expanduser(calcurse_config + "/caldav/hooks/")
+    oauth_file = os.path.expanduser(calcurse_config + "/caldav/oauth2_cred")
 
 # Parse command line arguments.
 parser = argparse.ArgumentParser('calcurse-caldav')
