@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2017 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2020 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -84,7 +84,8 @@
 #endif /* ENABLE_NLS */
 
 /* Paths configuration. */
-#define DIR_NAME         ".calcurse/"
+#define DIR_NAME         "calcurse/"
+#define DIR_NAME_LEGACY  ".calcurse/"
 #define TODO_PATH_NAME   "todo"
 #define APTS_PATH_NAME   "apts"
 #define CONF_PATH_NAME   "conf"
@@ -160,8 +161,11 @@
 #define KEYS_LABELEN        8	/* length of command description */
 #define KEYS_CMDS_PER_LINE  6	/* max number of commands per line */
 
-/* Register definitions. */
-#define REG_BLACK_HOLE 37
+/*
+ * Register definitions, see ui_day.c.
+ * Index 0-35 is used for '0'-'9' and 'a'-'z', see keys_get().
+ */
+#define REG_BLACK_HOLE 36
 
 /* Size of the hash table the note garbage collector uses. */
 #define NOTE_GC_HSIZE 1024
@@ -287,6 +291,7 @@ struct conf {
 	unsigned event_separator;
 	unsigned day_separator;
 	unsigned empty_appt_line;
+	char empty_day[BUFSIZ];
 	const char *editor;
 	const char *pager;
 	const char *mergetool;
@@ -296,6 +301,8 @@ struct conf {
 	char day_heading[BUFSIZ];	/* format for displaying heading in appts panel */
 };
 
+#define EMPTY_DAY_DEFAULT "--"
+
 /* Daemon-related configuration. */
 struct dmon_conf {
 	unsigned enable;	/* launch daemon automatically when exiting */
@@ -869,9 +876,9 @@ int display_help(const char *);
 int run_hook(const char *);
 
 /* ical.c */
-void ical_import_data(FILE *, FILE *, unsigned *, unsigned *, unsigned *,
-		      unsigned *, unsigned *, const char *, const char *,
-		      const char *, const char *, const char *);
+void ical_import_data(const char *, FILE *, FILE *, unsigned *, unsigned *,
+		      unsigned *, unsigned *, unsigned *, const char *,
+		      const char *, const char *, const char *, const char *);
 void ical_export_data(FILE *, int);
 
 /* io.c */
@@ -896,7 +903,7 @@ int io_check_file(const char *);
 int io_check_data_files(void);
 void io_startup_screen(int);
 void io_export_data(enum export_type, int);
-void io_import_data(enum import_type, const char *, const char *, const char *,
+void io_import_data(enum import_type, char *, const char *, const char *,
 		    const char *, const char *, const char *);
 struct io_file *io_log_init(void);
 void io_log_print(struct io_file *, int, const char *);
@@ -1111,6 +1118,7 @@ void ui_day_item_delete(unsigned);
 void ui_day_item_edit(void);
 void ui_day_item_pipe(void);
 void ui_day_item_repeat(void);
+void ui_day_item_cut(unsigned);
 void ui_day_item_cut_free(unsigned);
 void ui_day_item_copy(unsigned);
 void ui_day_item_paste(unsigned);
