@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2017 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2020 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -72,6 +72,27 @@ static void ical_export_footer(FILE *);
 static const char *ical_recur_type[RECUR_TYPES] =
     { "", "DAILY", "WEEKLY", "MONTHLY", "YEARLY" };
 
+/* Escape characters in field before printing */
+static void ical_format_line(FILE * stream, char * field, char * msg)
+{
+	char * p;
+
+	fputs(field, stream);
+	for (p = msg; *p; p++) {
+		switch (*p) {
+			case ',':
+			case ';':
+			case '\\':
+				fprintf(stream, "\\%c", *p);
+				break;
+			default:
+			fputc(*p, stream);
+                break;
+		}
+	}
+	fputc('\n', stream);
+}
+
 /* iCal alarm notification. */
 static void ical_export_valarm(FILE * stream)
 {
@@ -130,7 +151,7 @@ static void ical_export_recur_events(FILE * stream, int export_uid)
 			}
 		}
 
-		fprintf(stream, "SUMMARY:%s\n", rev->mesg);
+		ical_format_line(stream, "SUMMARY:", rev->mesg);
 
 		if (export_uid) {
 			char *hash = recur_event_hash(rev);
@@ -153,7 +174,7 @@ static void ical_export_events(FILE * stream, int export_uid)
 		date_sec2date_fmt(ev->day, ICALDATEFMT, ical_date);
 		fputs("BEGIN:VEVENT\n", stream);
 		fprintf(stream, "DTSTART;VALUE=DATE:%s\n", ical_date);
-		fprintf(stream, "SUMMARY:%s\n", ev->mesg);
+		ical_format_line(stream, "SUMMARY:", ev->mesg);
 
 		if (export_uid) {
 			char *hash = event_hash(ev);
@@ -209,7 +230,7 @@ static void ical_export_recur_apoints(FILE * stream, int export_uid)
 			}
 		}
 
-		fprintf(stream, "SUMMARY:%s\n", rapt->mesg);
+		ical_format_line(stream, "SUMMARY:", rapt->mesg);
 		if (rapt->state & APOINT_NOTIFY)
 			ical_export_valarm(stream);
 
@@ -244,7 +265,7 @@ static void ical_export_apoints(FILE * stream, int export_uid)
 				(apt->dur / MININSEC) % HOURINMIN,
 				apt->dur % MININSEC);
 		}
-		fprintf(stream, "SUMMARY:%s\n", apt->mesg);
+		ical_format_line(stream, "SUMMARY:", apt->mesg);
 		if (apt->state & APOINT_NOTIFY)
 			ical_export_valarm(stream);
 
@@ -271,7 +292,7 @@ static void ical_export_todo(FILE * stream, int export_uid)
 		if (todo->completed)
 			fprintf(stream, "STATUS:COMPLETED\n");
 		fprintf(stream, "PRIORITY:%d\n", todo->id);
-		fprintf(stream, "SUMMARY:%s\n", todo->mesg);
+		ical_format_line(stream, "SUMMARY:", todo->mesg);
 
 		if (export_uid) {
 			char *hash = todo_hash(todo);
@@ -284,26 +305,35 @@ static void ical_export_todo(FILE * stream, int export_uid)
 }
 
 /* Print a header to describe import log report format. */
-static void ical_log_init(FILE * log, int major, int minor)
+static void ical_log_init(const char *file, FILE * log, int major, int minor)
 {
 	const char *header =
 	    "+-------------------------------------------------------------------+\n"
 	    "| Calcurse icalendar import log.                                    |\n"
 	    "|                                                                   |\n"
-	    "| Items imported from icalendar file, version %d.%d                   |\n"
-	    "| Some items could not be imported, they are described hereafter.   |\n"
+	    "| Import from icalendar file                                        |\n"
+	    "|       %-60s|\n"
+	    "| version %d.%d at %s.                                  |\n"
+	    "|                                                                   |\n"
+	    "| Items which could not be imported are described below.            |\n"
 	    "| The log line format is as follows:                                |\n"
 	    "|                                                                   |\n"
 	    "|       TYPE [LINE]: DESCRIPTION                                    |\n"
 	    "|                                                                   |\n"
 	    "| where:                                                            |\n"
-	    "|  * TYPE represents the item type ('VEVENT' or 'VTODO')            |\n"
-	    "|  * LINE is the line in the input stream at which this item begins |\n"
-	    "|  * DESCRIPTION indicates why the item could not be imported       |\n"
+	    "|  * TYPE is the item type, 'VEVENT' or 'VTODO'                     |\n"
+	    "|  * LINE is the line in the import file where the item begins      |\n"
+	    "|  * DESCRIPTION explains why the item could not be imported        |\n"
 	    "+-------------------------------------------------------------------+\n\n";
 
+	char *date, *fmt;
+
+	asprintf(&fmt, "%s %s", DATEFMT(conf.input_datefmt), "%H:%M");
+	date = date_sec2date_str(now(), fmt);
 	if (log)
-		fprintf(log, header, major, minor);
+		fprintf(log, header, file, major, minor, date);
+	mem_free(fmt);
+	mem_free(date);
 }
 
 /*
@@ -410,8 +440,9 @@ ical_store_apoint(char *mesg, char *note, long start, long dur,
 }
 
 /*
- * Returns an allocated string representing the string given in argument once
- * unformatted.
+ * Returns an allocated string representing the argument string with escaped
+ * characters decoded, or NULL on error.
+ * The string is assumed to be the value part of a SUMMARY or DESCRIPTION line.
  */
 static char *ical_unformat_line(char *line)
 {
@@ -423,25 +454,29 @@ static char *ical_unformat_line(char *line)
 		switch (*p) {
 		case '\\':
 			switch (*(p + 1)) {
+			case 'N':
 			case 'n':
 				string_catf(&s, "%c", '\n');
 				p++;
 				break;
-			case 't':
-				string_catf(&s, "%c", '\t');
-				p++;
-				break;
+			case '\\':
 			case ';':
-			case ':':
 			case ',':
 				string_catf(&s, "%c", *(p + 1));
 				p++;
 				break;
 			default:
-				string_catf(&s, "%c", *p);
-				break;
+				mem_free(s.buf);
+				return NULL;
 			}
 			break;
+		case ',':
+		case ';':
+			/*
+			 * No list or field separator allowed.
+			 */
+			mem_free(s.buf);
+			return NULL;
 		default:
 			string_catf(&s, "%c", *p);
 			break;
@@ -458,13 +493,13 @@ ical_readline_init(FILE * fdi, char *buf, char *lstore, unsigned *ln)
 
 	*buf = *lstore = '\0';
 	if (fgets(lstore, BUFSIZ, fdi)) {
+		(*ln)++;
 		if ((eol = strchr(lstore, '\n')) != NULL) {
 			if (*(eol - 1) == '\r')
 				*(eol - 1) = '\0';
 			else
 				*eol = '\0';
 		}
-		(*ln)++;
 	}
 }
 
@@ -473,9 +508,9 @@ static int ical_readline(FILE * fdi, char *buf, char *lstore, unsigned *ln)
 	char *eol;
 
 	strncpy(buf, lstore, BUFSIZ);
-	(*ln)++;
 
 	while (fgets(lstore, BUFSIZ, fdi) != NULL) {
+		(*ln)++;
 		if ((eol = strchr(lstore, '\n')) != NULL) {
 			if (*(eol - 1) == '\r')
 				*(eol - 1) = '\0';
@@ -485,7 +520,6 @@ static int ical_readline(FILE * fdi, char *buf, char *lstore, unsigned *ln)
 		if (*lstore != SPACE && *lstore != TAB)
 			break;
 		strncat(buf, lstore + 1, BUFSIZ - strlen(buf) - 1);
-		(*ln)++;
 	}
 
 	if (feof(fdi)) {
@@ -675,6 +709,8 @@ static long ical_compute_rpt_until(long start, ical_rpt_t * rpt)
  */
 static char *ical_get_value(char *p)
 {
+	if (!(p && *p))
+		return NULL;
 	for (; *p != ':'; p++) {
 		if (*p == '"')
 			for (p++; *p != '"' && *p != '\0'; p++);
@@ -735,7 +771,7 @@ static ical_rpt_t *ical_read_rrule(FILE * log, char *rrulestr,
 	p = ical_get_value(rrulestr);
 	if (!p) {
 		ical_log(log, ICAL_VEVENT, itemline,
-			 _("recurrence rule malformed."));
+			 _("malformed recurrence line."));
 		(*noskipped)++;
 		return NULL;
 	}
@@ -815,7 +851,7 @@ static void ical_add_exc(llist_t * exc_head, long date)
  * This property defines the list of date/time exceptions for a
  * recurring calendar component.
  */
-static void
+static int
 ical_read_exdate(llist_t * exc, FILE * log, char *exstr,
 		 unsigned *noskipped, const int itemline)
 {
@@ -824,9 +860,9 @@ ical_read_exdate(llist_t * exc, FILE * log, char *exstr,
 	p = ical_get_value(exstr);
 	if (!p) {
 		ical_log(log, ICAL_VEVENT, itemline,
-			 _("recurrence exception dates malformed."));
+			 _("malformed exceptions line."));
 		(*noskipped)++;
-		return;
+		return 0;
 	}
 
 	while ((q = strchr(p, ',')) != NULL) {
@@ -839,6 +875,8 @@ ical_read_exdate(llist_t * exc, FILE * log, char *exstr,
 		p = ++q;
 	}
 	ical_add_exc(exc, ical_datetime2time_t(p, NULL));
+
+	return 1;
 }
 
 /* Return an allocated string containing the name of the newly created note. */
@@ -851,19 +889,20 @@ static char *ical_read_note(char *line, unsigned *noskipped,
 	p = ical_get_value(line);
 	if (!p) {
 		ical_log(log, item_type, itemline,
-			 _("description malformed."));
+			 _("malformed description line."));
 		(*noskipped)++;
 		return NULL;
 	}
 
 	notestr = ical_unformat_line(p);
-	if (notestr == NULL) {
-		ical_log(log, item_type, itemline,
-			 _("could not get entire item description."));
+	if (!notestr) {
+		ical_log(log, item_type, itemline, _("malformed description."));
 		(*noskipped)++;
 		return NULL;
 	} else if (strlen(notestr) == 0) {
 		mem_free(notestr);
+		ical_log(log, item_type, itemline, _("empty description."));
+		(*noskipped)++;
 		return NULL;
 	} else {
 		note = generate_note(notestr);
@@ -873,21 +912,33 @@ static char *ical_read_note(char *line, unsigned *noskipped,
 }
 
 /* Returns an allocated string containing the ical item summary. */
-static char *ical_read_summary(char *line)
+static char *ical_read_summary(char *line, unsigned *noskipped,
+			       ical_types_e item_type, const int itemline,
+			       FILE * log)
 {
 	char *p, *summary;
 
 	p = ical_get_value(line);
-	if (!p)
+	if (!p) {
+		ical_log(log, item_type, itemline, _("malformed summary line"));
+		(*noskipped)++;
 		return NULL;
+	}
 
 	summary = ical_unformat_line(p);
-	if (!summary)
+	if (!summary) {
+		ical_log(log, item_type, itemline, _("malformed summary."));
+		(*noskipped)++;
 		return NULL;
+	}
 
 	/* Event summaries must not contain newlines. */
-	for (p = strchr(summary, '\n'); p; p = strchr(p, '\n'))
-		*p = ' ';
+	if (strchr(summary, '\n')) {
+		ical_log(log, item_type, itemline, _("line break in summary."));
+		(*noskipped)++;
+		mem_free(summary);
+		return NULL;
+	}
 
 	return summary;
 }
@@ -898,7 +949,7 @@ ical_read_event(FILE * fdi, FILE * log, unsigned *noevents,
 		char *lstore, unsigned *lineno, const char *fmt_ev,
 		const char *fmt_rev, const char *fmt_apt, const char *fmt_rapt)
 {
-	const int ITEMLINE = *lineno;
+	const int ITEMLINE = *lineno - !feof(fdi);
 	ical_vevent_e vevent_type;
 	char *p;
 	struct {
@@ -929,12 +980,12 @@ ical_read_event(FILE * fdi, FILE * log, unsigned *noevents,
 			if (!vevent.mesg) {
 				ical_log(log, ICAL_VEVENT, ITEMLINE,
 					 _("could not retrieve item summary."));
-				goto cleanup;
+				goto skip;
 			}
 			if (vevent.start == 0) {
 				ical_log(log, ICAL_VEVENT, ITEMLINE,
 					 _("item start date is not defined."));
-				goto cleanup;
+				goto skip;
 			}
 
 			if (vevent_type == APPOINTMENT && vevent.dur == 0) {
@@ -945,7 +996,7 @@ ical_read_event(FILE * fdi, FILE * log, unsigned *noevents,
 				if (vevent.dur < 0) {
 					ical_log(log, ICAL_VEVENT, ITEMLINE,
 						_("item has a negative duration."));
-					goto cleanup;
+					goto skip;
 				}
 			}
 
@@ -974,7 +1025,7 @@ ical_read_event(FILE * fdi, FILE * log, unsigned *noevents,
 			case UNDEFINED:
 				ical_log(log, ICAL_VEVENT, ITEMLINE,
 					 _("item could not be identified."));
-				goto cleanup;
+				goto skip;
 				break;
 			}
 
@@ -985,50 +1036,58 @@ ical_read_event(FILE * fdi, FILE * log, unsigned *noevents,
 			p = ical_get_value(buf);
 			if (!p) {
 				ical_log(log, ICAL_VEVENT, ITEMLINE,
-					 _("event start time malformed."));
-				goto cleanup;
+					 _("malformed start time line."));
+				goto skip;
 			}
 
 			vevent.start = ical_datetime2time_t(p, &vevent_type);
 			if (!vevent.start) {
 				ical_log(log, ICAL_VEVENT, ITEMLINE,
 					 _("could not retrieve event start time."));
-				goto cleanup;
+				goto skip;
 			}
 		} else if (starts_with_ci(buf, "DTEND")) {
 			p = ical_get_value(buf);
 			if (!p) {
 				ical_log(log, ICAL_VEVENT, ITEMLINE,
-					 _("event end time malformed."));
-				goto cleanup;
+					 _("malformed end time line."));
+				goto skip;
 			}
 
 			vevent.end = ical_datetime2time_t(p, &vevent_type);
 			if (!vevent.end) {
 				ical_log(log, ICAL_VEVENT, ITEMLINE,
 					 _("could not retrieve event end time."));
-				goto cleanup;
+				goto skip;
 			}
 		} else if (starts_with_ci(buf, "DURATION")) {
 			vevent.dur = ical_dur2long(buf);
 			if (vevent.dur <= 0) {
 				ical_log(log, ICAL_VEVENT, ITEMLINE,
 					 _("item duration malformed."));
-				goto cleanup;
+				goto skip;
 			}
 		} else if (starts_with_ci(buf, "RRULE")) {
 			vevent.rpt = ical_read_rrule(log, buf, noskipped,
 					ITEMLINE);
+			if (!vevent.rpt)
+				goto cleanup;
 		} else if (starts_with_ci(buf, "EXDATE")) {
-			ical_read_exdate(&vevent.exc, log, buf, noskipped,
-					ITEMLINE);
+			if (!ical_read_exdate(&vevent.exc, log, buf, noskipped,
+					      ITEMLINE))
+				goto cleanup;
 		} else if (starts_with_ci(buf, "SUMMARY")) {
-			vevent.mesg = ical_read_summary(buf);
+			vevent.mesg = ical_read_summary(buf, noskipped,
+					ICAL_VEVENT, ITEMLINE, log);
+			if (!vevent.mesg)
+				goto cleanup;
 		} else if (starts_with_ci(buf, "BEGIN:VALARM")) {
 			skip_alarm = vevent.has_alarm = 1;
 		} else if (starts_with_ci(buf, "DESCRIPTION")) {
 			vevent.note = ical_read_note(buf, noskipped,
 					ICAL_VEVENT, ITEMLINE, log);
+			if (!vevent.note)
+				goto cleanup;
 		}
 	}
 
@@ -1036,8 +1095,10 @@ ical_read_event(FILE * fdi, FILE * log, unsigned *noevents,
 		 _("The ical file seems to be malformed. "
 		   "The end of item was not found."));
 
-cleanup:
+skip:
+	(*noskipped)++;
 
+cleanup:
 	if (vevent.note)
 		mem_free(vevent.note);
 	if (vevent.mesg)
@@ -1045,14 +1106,13 @@ cleanup:
 	if (vevent.rpt)
 		mem_free(vevent.rpt);
 	LLIST_FREE(&vevent.exc);
-	(*noskipped)++;
 }
 
 static void
 ical_read_todo(FILE * fdi, FILE * log, unsigned *notodos, unsigned *noskipped,
 	       char *buf, char *lstore, unsigned *lineno, const char *fmt_todo)
 {
-	const int ITEMLINE = *lineno;
+	const int ITEMLINE = *lineno - !feof(fdi);
 	struct {
 		char *mesg, *note;
 		int priority;
@@ -1092,16 +1152,23 @@ ical_read_todo(FILE * fdi, FILE * log, unsigned *notodos, unsigned *noskipped,
 				ical_log(log, ICAL_VTODO, ITEMLINE,
 					 _("item priority is invalid "
 					  "(must be between 0 and 9)."));
+				goto skip;
 			}
 		} else if (starts_with_ci(buf, "STATUS:COMPLETED")) {
 			vtodo.completed = 1;
 		} else if (starts_with_ci(buf, "SUMMARY")) {
-			vtodo.mesg = ical_read_summary(buf);
+			vtodo.mesg =
+				ical_read_summary(buf, noskipped, ICAL_VTODO,
+						  ITEMLINE, log);
+			if (!vtodo.mesg)
+				goto cleanup;
 		} else if (starts_with_ci(buf, "BEGIN:VALARM")) {
 			skip_alarm = 1;
 		} else if (starts_with_ci(buf, "DESCRIPTION")) {
 			vtodo.note = ical_read_note(buf, noskipped, ICAL_VTODO,
 					ITEMLINE, log);
+			if (!vtodo.note)
+				goto cleanup;
 		}
 	}
 
@@ -1109,17 +1176,18 @@ ical_read_todo(FILE * fdi, FILE * log, unsigned *notodos, unsigned *noskipped,
 		 _("The ical file seems to be malformed. "
 		   "The end of item was not found."));
 
+skip:
+	(*noskipped)++;
 cleanup:
 	if (vtodo.note)
 		mem_free(vtodo.note);
 	if (vtodo.mesg)
 		mem_free(vtodo.mesg);
-	(*noskipped)++;
 }
 
 /* Import calcurse data. */
 void
-ical_import_data(FILE * stream, FILE * log, unsigned *events,
+ical_import_data(const char *file, FILE * stream, FILE * log, unsigned *events,
 		 unsigned *apoints, unsigned *todos, unsigned *lines,
 		 unsigned *skipped, const char *fmt_ev, const char *fmt_rev,
 		 const char *fmt_apt, const char *fmt_rapt,
@@ -1134,10 +1202,9 @@ ical_import_data(FILE * stream, FILE * log, unsigned *events,
 		  _("Warning: ical header malformed or wrong version number. "
 		   "Aborting..."));
 
-	ical_log_init(log, major, minor);
+	ical_log_init(file, log, major, minor);
 
 	while (ical_readline(stream, buf, lstore, lines)) {
-		(*lines)++;
 		if (starts_with_ci(buf, "BEGIN:VEVENT")) {
 			ical_read_event(stream, log, events, apoints,
 					skipped, buf, lstore, lines, fmt_ev,
