@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2017 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2020 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -136,24 +136,52 @@ unsigned io_fprintln(const char *fname, const char *fmt, ...)
 /*
  * Initialization of data paths. The cfile argument is the variable
  * which contains the calendar file. If none is given, then the default
- * one (~/.calcurse/apts) is taken. If the one given does not exist, it
- * is created.
+ * one (~/.local/share/calcurse/apts) is taken. If the one given does not exist,
+ * it is created.
  * The datadir argument can be used to specify an alternative data root dir.
  * The confdir argument can be used to specify an alternative configuration dir.
+ * If ~/.calcurse exists, it will be used instead for backward compatibility.
  */
 void io_init(const char *cfile, const char *datadir, const char *confdir)
 {
-	if (!datadir) {
-		if (!(datadir = getenv("HOME")))
-			datadir = ".";
-		asprintf(&path_ddir, "%s%s", datadir, "/" DIR_NAME);
-	} else
-		asprintf(&path_ddir, "%s%s", datadir, "/");
+	char* home_dir = getenv("HOME");
+	char* legacy_dir = NULL;
 
-	if (!confdir)
-		asprintf(&path_cdir, "%s%s", path_ddir, "/");
+	if (home_dir) {
+		asprintf(&legacy_dir, "%s%s", home_dir, "/" DIR_NAME_LEGACY);
+		if (!io_dir_exists(legacy_dir)) {
+			mem_free(legacy_dir);
+			legacy_dir = NULL;
+		}
+	}
+
+	if (datadir)
+		asprintf(&path_ddir, "%s%s", datadir, "/");
+	else if (legacy_dir)
+		path_ddir = mem_strdup(legacy_dir);
+	else if ((path_ddir = getenv("XDG_DATA_HOME")))
+		asprintf(&path_ddir, "%s%s", path_ddir, "/" DIR_NAME);
+	else if (home_dir)
+		asprintf(&path_ddir, "%s%s", home_dir, "/.local/share/" DIR_NAME);
 	else
+		path_ddir = mem_strdup("./." DIR_NAME);
+
+
+	if (confdir)
 		asprintf(&path_cdir, "%s%s", confdir, "/");
+	else if (datadir)
+		path_cdir = mem_strdup(path_ddir);
+	else if (legacy_dir)
+		path_cdir = mem_strdup(legacy_dir);
+	else if ((path_cdir = getenv("XDG_CONFIG_HOME")))
+		asprintf(&path_cdir, "%s%s", path_cdir, "/" DIR_NAME);
+	else if (home_dir)
+		asprintf(&path_cdir, "%s%s", home_dir, "/.config/" DIR_NAME);
+	else
+		path_cdir = mem_strdup("./." DIR_NAME);
+
+	if (legacy_dir)
+		mem_free(legacy_dir);
 
 	/* Data files */
 	if (cfile) {
@@ -1053,19 +1081,42 @@ int io_check_dir(const char *dir)
 	if (read_only)
 		return -1;
 
+	char *path = mem_strdup(dir);
+	char *index;
+
+	int existed = 1, failed = 0;
 	errno = 0;
-	if (mkdir(dir, 0700) != 0) {
-		if (errno != EEXIST) {
-			fprintf(stderr,
-				_("FATAL ERROR: could not create %s: %s\n"),
-				dir, strerror(errno));
-			exit_calcurse(EXIT_FAILURE);
-		} else {
-			return 1;
+	for (index = path + 1; *index; index++) {
+		if (*index == '/') {
+			*index = '\0';
+			if (mkdir(path, 0700) != 0) {
+				if (errno != EEXIST) {
+					failed = 1;
+					break;
+				}
+			} else {
+				existed = 0;
+			}
+			*index = '/';
 		}
-	} else {
-		return 0;
 	}
+
+	if (!failed && mkdir(path, 0700) != 0) {
+		if (errno != EEXIST)
+			failed = 1;
+	} else {
+		existed = 0;
+	}
+
+	if(failed) {
+		fprintf(stderr,
+			_("FATAL ERROR: could not create %s: %s\n"),
+			path, strerror(errno));
+		exit_calcurse(EXIT_FAILURE);
+	}
+
+	mem_free(path);
+	return existed;
 }
 
 unsigned io_dir_exists(const char *path)
@@ -1114,11 +1165,15 @@ int io_check_file(const char *file)
  * Checks if data files exist. If not, create them.
  * The following structure has to be created:
  *
- *   <datadir>   <configdir> (default for both: $HOME/.calcurse/)
+ *   <datadir>   <configdir>
  *        |             |
  *        |__ apts      |___ conf
  *        |__ todo      |___ keys
  *        |__ notes/    |___ hooks/
+ *
+ * Defaults:
+ * - datadir: $XDG_DATA_HOME/calcurse (~/.local/share/calcurse)
+ * - configdir: $XDG_CONFIG_HOME/calcurse (~/.config/calcurse)
  */
 int io_check_data_files(void)
 {
@@ -1192,31 +1247,29 @@ void io_export_data(enum export_type type, int export_uid)
 	}
 }
 
-static FILE *get_import_stream(enum import_type type)
+static FILE *get_import_stream(enum import_type type, char **stream_name)
 {
 	FILE *stream = NULL;
-	char *stream_name;
 	const char *ask_fname =
 	    _("Enter the file name to import data from:");
 	const char *wrong_file =
 	    _("The file cannot be accessed, please enter another file name.");
 	const char *press_enter = _("Press [ENTER] to continue.");
 
-	stream_name = mem_malloc(BUFSIZ);
-	memset(stream_name, 0, BUFSIZ);
+	*stream_name = mem_malloc(BUFSIZ);
+	memset(*stream_name, 0, BUFSIZ);
 	while (stream == NULL) {
 		status_mesg(ask_fname, "");
-		if (updatestring(win[STA].p, &stream_name, 0, 1)) {
-			mem_free(stream_name);
+		if (updatestring(win[STA].p, stream_name, 0, 1)) {
+			mem_free(*stream_name);
 			return NULL;
 		}
-		stream = fopen(stream_name, "r");
+		stream = fopen(*stream_name, "r");
 		if (stream == NULL) {
 			status_mesg(wrong_file, press_enter);
 			keys_wait_for_any_key(win[KEY].p);
 		}
 	}
-	mem_free(stream_name);
 
 	return stream;
 }
@@ -1227,7 +1280,7 @@ static FILE *get_import_stream(enum import_type type)
  * A temporary log file is created in /tmp to store the import process report,
  * and is cleared at the end.
  */
-void io_import_data(enum import_type type, const char *stream_name,
+void io_import_data(enum import_type type, char *stream_name,
 		    const char *fmt_ev, const char *fmt_rev,
 		    const char *fmt_apt, const char *fmt_rapt,
 		    const char *fmt_todo)
@@ -1254,7 +1307,7 @@ void io_import_data(enum import_type type, const char *stream_name,
 			 "Aborting..."));
 		break;
 	case UI_CURSES:
-		stream = get_import_stream(type);
+		stream = get_import_stream(type, &stream_name);
 		break;
 	default:
 		EXIT(_("FATAL ERROR: wrong import mode"));
@@ -1274,7 +1327,7 @@ void io_import_data(enum import_type type, const char *stream_name,
 	}
 
 	if (type == IO_IMPORT_ICAL)
-		ical_import_data(stream, log->fd, &stats.events,
+		ical_import_data(stream_name, stream, log->fd, &stats.events,
 				 &stats.apoints, &stats.todos,
 				 &stats.lines, &stats.skipped, fmt_ev, fmt_rev,
 				 fmt_apt, fmt_rapt, fmt_todo);
@@ -1282,6 +1335,10 @@ void io_import_data(enum import_type type, const char *stream_name,
 	if (stream != stdin)
 		file_close(stream, __FILE_POS__);
 
+	if (ui_mode == UI_CURSES &&
+	    (stats.apoints > 0 || stats.events > 0 || stats.todos > 0))
+		io_set_modified();
+
 	asprintf(&stats_str[0], ngettext("%d app", "%d apps", stats.apoints),
 		 stats.apoints);
 	asprintf(&stats_str[1],
@@ -1321,7 +1378,10 @@ void io_import_data(enum import_type type, const char *stream_name,
 	mem_free(stats_str[1]);
 	mem_free(stats_str[2]);
 	mem_free(stats_str[3]);
-	io_log_free(log);
+	if (ui_mode == UI_CURSES)
+		mem_free(stream_name);
+	if (!stats.skipped)
+		io_log_free(log);
 }
 
 struct io_file *io_log_init(void)
