@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2017 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2020 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -297,8 +297,6 @@ enum key keys_get(WINDOW *win, int *count, int *reg)
 				*reg = ch - '1' + 1;
 			} else if (ch >= 'a' && ch <= 'z') {
 				*reg = ch - 'a' + 10;
-			} else if (ch == '_') {
-				*reg = REG_BLACK_HOLE;
 			}
 			ch = keys_wgetch(win);
 		}
