@@ -1,7 +1,7 @@
 /*
  * Calcurse - text-based organizer
  *
- * Copyright (c) 2004-2017 calcurse Development Team <misc@calcurse.org>
+ * Copyright (c) 2004-2020 calcurse Development Team <misc@calcurse.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -112,7 +112,7 @@ void free_user_data(void)
 	apoint_llist_free();
 	recur_apoint_llist_free();
 	recur_event_llist_free();
-	for (i = 0; i <= 37; i++)
+	for (i = 0; i <= REG_BLACK_HOLE; i++)
 		ui_day_item_cut_free(i);
 	todo_free_list();
 	notify_free_app();
